from logging.handlers import MemoryHandler


class CustomMemoryHandler(MemoryHandler):

    def emit(self, record):
        """
        Emit a record or append it to the buffer
        """
        if self.shouldFlush():
            self.flush()
        else:
            self.buffer.append(record)

    def shouldFlush(self):
        """
        Flush only when buffer is full
        """
        return len(self.buffer) >= self.capacity

    def flush(self) -> None:
        """
        Caches a number of logs in memory, and flushes all of them in one go.

        Example:
        capacity = 10
        memory_handler = CustomMemoryHandler(capacity,
                                             flushLevel=logging.WARNING,
                                             target=stream_handler)
        """
        self.acquire()
        try:
            if self.target:
                # Send to target all the logs
                for log in self.buffer:
                    self.target.handle(log)

                self.buffer.clear()

        finally:
            self.release()



from abc import ABC, abstractmethod
from logging import DEBUG, Logger, Handler


class HandlerFactoryAbstract(ABC):

    @abstractmethod
    def create_handler(self,
                       logging_level: int = DEBUG
                       ) -> Handler:
        ...

    @abstractmethod
    def add_handler(self, logger: Logger, logging_level: int = DEBUG) -> Logger:
        ...



from logging import DEBUG, Formatter, Logger, StreamHandler as StreamHandlerLogging
import sys

from .abstract import HandlerFactoryAbstract


class StreamHandlerFactory(HandlerFactoryAbstract):

    def __init__(self, formatter: Formatter) -> None:
        self.formatter = formatter

    def create_handler(self, logging_level: int = DEBUG) -> StreamHandlerLogging:
        stream_handler = StreamHandlerLogging(sys.stdout)

        stream_handler.setLevel(logging_level)
        stream_handler.setFormatter(self.formatter)
        return stream_handler

    def add_handler(self, logger: Logger, logging_level: int = DEBUG) -> Logger:
        stream_handler = self.create_handler(logging_level)
        logger.addHandler(stream_handler)

        return logger



from logging import DEBUG, Logger, Handler

from logger_builder.handler import CustomMemoryHandler
from .abstract import HandlerFactoryAbstract


class MemoryHandlerFactory(HandlerFactoryAbstract):

    def __init__(self,
                 target_handler_factory: HandlerFactoryAbstract,
                 logging_buffer_size: int = 5,
                 ) -> None:
        self.target_handler_factory = target_handler_factory
        self.logging_buffer_size = logging_buffer_size

    def create_handler(self,
                       logging_level: int = DEBUG
                       ) -> CustomMemoryHandler:

        target_handler = self.target_handler_factory.create_handler()
        memory_handler = CustomMemoryHandler(self.logging_buffer_size,
                                             logging_level,
                                             target_handler,
                                             )

        return memory_handler

    def add_handler(self, logger: Logger, logging_level: int = DEBUG) -> Logger:
        memory_handler = self.create_handler(logging_level)
        logger.addHandler(memory_handler)
        return logger



from datetime import datetime
from logging import DEBUG, Formatter, Logger, FileHandler as FileHandlerLogging
from typing import TYPE_CHECKING

from .abstract import HandlerFactoryAbstract

if TYPE_CHECKING:
    from pathlib import Path


class FileHandlerFactory(HandlerFactoryAbstract):

    def __init__(self,
                 log_file_path: 'Path',
                 log_file_name: str,
                 formatter: Formatter
                 ) -> None:
        self.log_file_path = log_file_path
        self.log_file_name = log_file_name
        self.formatter = formatter

    def create_handler(self, logging_level: int = DEBUG) -> FileHandlerLogging:
        file_name = f'{datetime.now().date()}_{self.log_file_name}.log'
        file_handler = FileHandlerLogging(self.log_file_path / file_name)

        file_handler.setLevel(logging_level)
        file_handler.setFormatter(self.formatter)

        return file_handler

    def add_handler(self, logger: Logger, logging_level: int = DEBUG) -> Logger:
        file_handler = self.create_handler(logging_level)

        logger.addHandler(file_handler)

        return logger



from pathlib import Path
import setuptools


def load_description_from_file(file_name: str, long_description: str) -> str:
    try:
        if Path(file_name).is_file():
            with open(file_name, "r", encoding="utf-8") as fh:
                long_description_partial = fh.read()
        long_description += "\n\n"
        long_description += long_description_partial
    except Exception as e:
        pass

    return long_description


long_description = ""
for description_file in ["README.md", "CHANGELOG.md"]:
    long_description = load_description_from_file(description_file, long_description)



setuptools.setup(
    name="logger_builder",
    version="0.0.3",
    author="Ludwik Bielczynski",
    author_email="ludwik.bielczynski@gmail.com",
    description="Logger builder to simplify creation of the loggers",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/LudwikBielczynski/logger_builder.git",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    packages=setuptools.find_packages(),
    python_requires=">=3.6"
)



from abc import ABC, abstractmethod
from logging import Logger, DEBUG, getLogger
from typing import Sequence

from .handler.factory import HandlerFactoryAbstract


class LoggerBuilderAbstract(ABC):
    def __init__(self, handler_factories: Sequence[HandlerFactoryAbstract]) -> None:
        self._handler_factories = handler_factories

    def reset(self) -> None:
        self._handler_factories = []

    @abstractmethod
    def create_logger(self, logger_name: str, logging_level: int = DEBUG) -> Logger:
        """Method implemented in concrete class returning a logger"""


class LoggerBuilder(LoggerBuilderAbstract):
    def create_logger(self, logger_name: str, logging_level: int = DEBUG) -> Logger:
        logger = getLogger(logger_name)
        logger.setLevel(logging_level)

        for handler in self._handler_factories:
            logger = handler.add_handler(logger)

        return logger



from logging import Formatter


def create_formatter(
    levelname: bool = True,
    process: bool = True,
    name: bool = True,
    funcName: bool = True,
    lineno: bool = True,
    datetime_format: str = "%Y-%m-%d %H:%M:%S",
    simple_description: bool = False,
) -> Formatter:
    # Create a formatter with only date and message logs
    if simple_description:
        levelname = False
        process = False
        name = False
        funcName = False
        lineno = False

    logs_format = '%(asctime)s.%(msecs).0f'

    def include_info(logs_format: str,
                     info_name: str,
                     prefix: str = ' ',
                     print_format: str = 's',
                     should_include: bool = True
                     ) -> str:
        """Include some additional info if needed"""
        if should_include:
            logs_format += f'{prefix}%({info_name}){print_format}'

        return logs_format

    prefix_map = {'levelname': ' ',
                  'process': ' PID:',
                  'name': ' ',
                  'funcName': '.',
                  'lineno': ':',
                  }
    print_format_map = {'levelname': 's',
                        'process': 'd',
                        'name': 's',
                        'funcName': 's',
                        'lineno': 'd',
                        }
    for info_name in ['levelname', 'process', 'name', 'funcName', 'lineno']:
        should_include = locals()[info_name]
        logs_format = include_info(logs_format,
                                   info_name,
                                   prefix_map[info_name],
                                   print_format_map[info_name],
                                   should_include,
                                   )

    logs_format = include_info(logs_format, 'message', ' ')

    formatter = Formatter(fmt=logs_format, datefmt=datetime_format)
    return formatter
